import sys
import datetime
import json
from threading import Lock
import asyncio
import time

import argparse
from .promise_set import PromiseSet

from .enums import AgentState
from . import types_pb2 as pb2
import google.protobuf.json_format as json_format
from google.protobuf.timestamp_pb2 import Timestamp

#from blackboard_grpc_client import BlackboardGRPCClient
#client_type = BlackboardGRPCClient

from .blackboard_http_client import BlackboardHTTPClient
client_type = BlackboardHTTPClient

# ----------------------------------------
# Agent base class
# Inheritors must implement self.run()
# ----------------------------------------
class Agent(client_type):

    bb_client = None

    is_fullfilled = asyncio.Event()
    blackboard_remote_addr = ""
    last_msg_idx = 0
    stopped = False
    mutex = Lock()
    promises = None
    
    def __init__(self, spec, bb_addr):
        self.spec = spec
        super().__init__(bb_addr)
        self.blackboard_remote_addr = bb_addr
        self.promises = PromiseSet(spec)
    
    def attribute(self, name):
        try:
            return self.spec.attributes[name]
        except:
            return None
        
    # Primary AgentHTTP interface that users should employ when writing
    # their own agents
    def log(self, severity, message):
        log_message = pb2.Message(log=pb2.Log(severity=severity, message = message))
        self.send_message(log_message)
        
    def post_status_update(self, state, message):
        #status_update = {"state": str(state), "message": update}
        status_update = pb2.Message(statusUpdate=pb2.StatusUpdate(state=state, message=message))
        self.send_message(status_update)
        
    def post_result(self, res_name: str, res_type: str, data: dict):
        # Convert the data dictionary to json
        json_data = json.dumps(data)
        
        promise = pb2.Promise(
            name=res_name,
            type=res_type,
            state=pb2.Promise.PromiseState.Final,
            data=json_data.encode("utf-8"),
            source=self.spec.name,
        )

        result = pb2.Result(
            type=pb2.Result.ResultType.Result,
            promise=promise,
        )

        msg = pb2.Message(result=result)
        
        self.send_message(msg)
        
    def post_partial_result(self, res_name: str, res_type: str, data: dict):
        json_data = json.dumps(data)
        
        promise = pb2.Promise(
            name=res_name,
            type=res_type,
            state=pb2.Promise.PromiseState.Fulfilled,
            data=json_data.encode("utf-8"),
            source=self.spec.name,
        )

        result = pb2.Result(
            type=pb2.Result.ResultType.PartialResult,
            promise=promise,
        )
        
        msg = pb2.Message(result=result)
        
        self.send_message(msg)

    def post_no_result(self, res_name: str, res_type: str, data: dict):
        json_data = json.dumps(data)
        
        promise = pb2.Promise(
            name=res_name,
            type=res_type,
            state=pb2.Promise.PromiseState.Empty,
            data=json_data.encode("utf-8"),
            source=self.spec.name,
        )

        result = pb2.Result(
            type=pb2.Result.ResultType.NoResult,
            promise=promise,
        )
        
        msg = pb2.Message(result=result)
        
        self.send_message(msg)
        
    # Users should not generally need to call these methods directly, as
    # they are handled by the base class, however they are available if needed
    def send_ping(self):
        self.send_message(pb2.Message(ping={}))

    def say_hello(self):
        msg = pb2.Message(hello={})
        self.send_message(msg)

    def say_goodbye(self):
        msg = pb2.Message(goodbye={})
        self.send_message(msg)

    # send_messages sends a message to the configured blackboard (via the HTTP
    # interface).  In general, users should try  to prefer the higher-level methods
    # (such as post_status_update, log, post_result, etc.)
    #
    # msg_type should be one of those specified in message_types.py
    # data should be the marshaled JSON string (not bytes) of your message content
    # send_message is inherited from the Blackboard*Client (), hence the _ prefix
    # We also want to discourage folks from using the _send_message method directly
    # as this is not really our intended interface. It should only be used in advanced
    # use cases that are not currently supported by SM.
    def send_message(self, msg):
        msg.source = self.spec.name
        msg.timeSent.GetCurrentTime()
        self._send_message(msg)
        
    async def start(self):
        self.say_hello()
        await asyncio.gather(
            self._ping_loop(),
            self._message_thread(),
            self.run(),
        )

    def launch(self):
        asyncio.run(self.start())
        self.say_goodbye()

    # User entrypoint code
    async def await_data(self):
        self.post_status_update(AgentState.AWAITING_PROMISE, "awaiting {} promises".format(len(self.promises.promises)))
        await self.is_fullfilled.wait()
        
    async def run(self):
        raise NotImplementedError()
    
    async def _ping_loop(self):
        while not self._stopped():
            await asyncio.sleep(self.spec.pingInterval_ms/1000)
            self.send_ping()
        return False

    async def _message_thread(self):
        while not self._stopped():

            await asyncio.sleep(self.spec.pollInterval_ms/1000 + 0.5)
            
            msgs = self._get_messages(self.last_msg_idx+1) # list of messages
            
            if len(msgs) > 0:
                for msg in msgs:
                    msg_type = msg.WhichOneof('contents')

                    if msg_type == "result":

                        # check if the result fulfills any of our promises
                        attr_name, exists = self.promises.contains(msg.result.promise)
                        
                        if exists:
                            self.promises.promises[attr_name] = msg.result.promise

                        if self.promises.all_fulfilled():
                            self.is_fullfilled.set()
                            
                    elif msg_type == "halt":
                        self.stop()
                        
                    self.last_msg_idx = msg.index
                    #self.last_msg_idx = msg["index"]
                    
        return False
    
    def stop(self):
        with self.mutex:
            self.stopped=True
            
    def _stopped(self):
        with self.mutex:
            return self.stopped
        
# This main runs some tests 
if __name__=="__main__":

    bb_addr = sys.argv[1]

    spec = pb2.AgentSpec(
        name="base-agent",
        pingInterval_ms=1000,
        pollInterval_ms=1000)

    a = AgentHTTP(spec, bb_addr)
    
    a.say_hello()
    a.send_ping()
    a.post_status_update(pb2.StatusUpdate.AgentState.Working, "starting some processing")
    a.log(pb2.Log.Severity.Info, "hello world")
    a.post_result("result_name", "result_type", {"hello":"world"})
    a.post_partial_result("partial_result_name", "partial_result_type", {"hello":"world"})
    a.post_no_result("no_result_name", "no_result_type", {"hello":"world"})
    print(a._get_messages(3))
    a.say_goodbye()

    #asyncio.run(a.start())
    


import json
import requests
from google.protobuf import json_format

from . import types_pb2 as pb2

class BlackboardHTTPClient():
    def __init__(self, addr):
        # instantiate a channel
        # bind the client and the server
        self.addr = addr
        #self.channel = grpc.insecure_channel(self.addr)
        #self.stub = pb2_grpc.BlackboardHandlerStub(self.channel)

    def _get_messages(self, starting_at, filter_pings = True):
        url = "http://{}/blackboard/{}".format(self.addr, starting_at)
        r = requests.get(url)
        msgs = r.json() # iterable list of messages

        # Translate from dictionary 
        msg_stack = json_format.ParseDict(msgs, pb2.MessageStack())
        return msg_stack.messages

    def _send_message(self, msg):
        url = "http://{}/message".format(self.addr)
        msg_json = json_format.MessageToJson(msg)
        r = requests.post(url, data=msg_json.encode('utf-8'))


import sys
from message_types import *
import datetime
import urllib.request, urllib.parse
import json
from threading import Lock
import asyncio
import time

import grpc
import types_pb2 as pb2
import types_pb2_grpc as pb2_grpc

def HTTPGet(url):
    request = urllib.request.Request(url)
    with urllib.request.urlopen(request) as f:
        return f.read().decode('utf-8')

def HTTPPost(url, msg):
    
    #data = urllib.parse.urlencode(data)
    data_str = msg.toJSON()
    data = data_str.encode("utf-8")
    request = urllib.request.Request(url)

    print("Posting data to url: ", url, data)
    with urllib.request.urlopen(request, data) as f:
        return f.read().decode('utf-8')
    
class Agent(AgentSpec):
    blackboard_remote_addr = ""
    last_msg_idx = 0
    stopped = False
    mutex = Lock()
    
    def __init__(self, spec, bb_addr):

        channel = grpc.insecure_channel('localhost:50051')
        stub = route_guide_pb2_grpc.RouteGuideStub(channel)

        
        self = spec
        self.blackboard_remote_addr = bb_addr
        self.start()
    
    # Primary Agent interface that users should employ when writing
    # their own agents
    def log(self, severity: LogSeverity, message: str):
        log_message = {"severity": str(severity), "message": message}
        log_string = json.dumps(log_message)
        self.send_message(MessageType.LOG, log_string)

    def post_status_update(self, state: AgentState, update: str):
        
        status_update = {"state": str(state), "message": update}
        update_string = json.dumps(status_update)
        self.send_message(MessageType.STATUSUPDATE, update_string)

    def post_result(self, res_name: str, res_type: str, data: dict):
        json_data = json.dumps(data)
        
        promise = {
            "dataidentifier": res_name,
            "promisetype": res_type,
            "source": self.Name,
            "data": json_data,
        }

        json_promise = json.dumps(promise)
        
        self.send_message(MessageType.RESULT, json_promise)
    
    def post_partial_result(self, res_name: str, res_type: str, data: dict):
        json_data = json.dumps(data)
        
        promise = {
            "dataidentifier": res_name,
            "promisetype": res_type,
            "source": self.Name,
            "data": json_data,
        }

        json_promise = json.dumps(promise)
        
        self.send_message(MessageType.PARTIALRESULT, json_promise)
        
    # Users should not generally need to call these methods directly, as
    # they are handled by the base class, however they are available if needed
    def send_ping(self):
        self.send_message(MessageType.PING, "")
    
    def say_hello(self):
        self.send_message(MessageType.HELLO, "")

    def say_goodbye(self):
        self.send_message(MessageType.GOODBYE, "")

    # send_messages sends a message to the configured blackboard (via the HTTP
    # interface).  In general, users should try  to prefer the higher-level methods
    # (such as post_status_update, log, post_result, etc.)
    #
    # msg_type should be one of those specified in message_types.py
    # data should be the marshaled JSON string (not bytes) of your message content
    def send_message(self, msg_type: MessageType, data: str):
        msg = Message(msg_type, data)
        msg.Type = msg_type
        msg.Source = self.Name
        msg.Data = data.encode("utf-8")
        msg.TimeSent = datetime.datetime.now(datetime.timezone.utc)
        self._post_message_http(msg)

    def _post_message_http(self, msg: Message):
        url = "http://" + self.blackboard_remote_addr + "/message"
        HTTPPost(url, msg)
        
    def _get_messages_http(self):
        start_idx = self.last_msg_idx + 1
        url = "http://" + self.blackboard_remote_addr + "/blackboard/" +  str(start_idx)
        r = HTTPGet(url)
        
        return json.loads(r)

    async def start(self):        
        await asyncio.gather(
            self._ping_loop(),
            self._message_thread(),
        )
        
    async def _ping_loop(self):
        while not self._stopped():
            await asyncio.sleep(self.PingInterval_ms/1000)
            self.send_ping()
        return False

    async def _message_thread(self):
        while not self._stopped():
            await asyncio.sleep(self.PollInterval_ms/1000 + 0.5)
            
            msgs = self._get_messages_http()# list of messages
            if msgs != None:
                for msg in msgs:
                    self.last_msg_idx = msg["index"]
            
        return False
            
    def _stopped(self):
        with self.mutex:
            return self.stopped
        
# This main runs some tests 
if __name__=="__main__":

    a = Agent(sys.argv[1], "localhost:8080")
    #a = Agent("python-baby!", "localhost:8080")
    
    #a._get_messages_http()
    #a.say_hello()
    #a.send_ping()
    #a.post_status_update(AgentState.WORKING, "starting some processing")
    #a.log(LogSeverity.INFO, "some logging")
    #a.post_result("result_name", "result_type", '{"hello":"world"}')
    #a.say_goodbye()

    asyncio.run(a.start(), a.run())
    


import sys
import re
from .enums import PromiseState
from .types_pb2 import AgentSpec, Promise

def parse_promise(promise_string: str):
    result = re.search("Promise\(\s*(\w+)\s+as\s+(\w+)\s+from\s+(\w+)\s*\)", promise_string)
    if result == None:
        return
    
    p = Promise(name=result.group(1),
                type=result.group(2),
                source=result.group(3),
                state=PromiseState.PENDING,
                data=None)
    
    return p

def parse_promises(spec: AgentSpec):
    promises = {}

    # Iterate over attributes, extracting promises and saving
    for attr in spec.attributes:
        attr_val = spec.attributes[attr]
        p = parse_promise(attr_val)
        
        if p == None:
            continue
        
        promises[attr]= p
    
    return promises

class PromiseSet():
    
    promises = {}
    
    def __init__(self, spec: AgentSpec):
        self.promises = parse_promises(spec)
        
    def all_fulfilled(self):
        for name, promise in self.promises.items():
            if promise.state == PromiseState.PENDING:
                return False

        return True

    def contains(self, promise):
        for name, curr_prom in self.promises.items():
            data_id_match = (promise.name == curr_prom.name)
            data_type_match = (promise.type == curr_prom.type)
            source_match = (promise.source == curr_prom.source)

            if data_type_match and data_id_match and source_match:
                return (name, True)

        return ("", False)
    
if __name__ == "__main__":

    spec = AgentSpec(attributes = {
        "second": "Promise( hello as hellotype from   silly_goose )",
        "notpromise":"10"})
    
    promise_set = PromiseSet(spec)
    if promise_set.all_fulfilled():
        print("FAIL: promises are not fulfilled")

    if len(promise_set.promises) > 1:
        print("FAIL: too many promises")

    # "fulfill" the promise
    promise_set.promises["second"].state = PromiseState.FULFILLED

    if not promise_set.all_fulfilled:
        print("FAIL: promises should be fulfilled")



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import types_pb2 as types__pb2


class BlackboardHandlerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetMessages = channel.unary_unary(
                '/BlackboardHandler/GetMessages',
                request_serializer=types__pb2.GetMessagesRequest.SerializeToString,
                response_deserializer=types__pb2.MessageStack.FromString,
                )
        self.SendMessage = channel.unary_unary(
                '/BlackboardHandler/SendMessage',
                request_serializer=types__pb2.Message.SerializeToString,
                response_deserializer=types__pb2.Ack.FromString,
                )
        self.OpenMessageStream = channel.stream_stream(
                '/BlackboardHandler/OpenMessageStream',
                request_serializer=types__pb2.Message.SerializeToString,
                response_deserializer=types__pb2.Message.FromString,
                )


class BlackboardHandlerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetMessages(self, request, context):
        """Push/pull API 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendMessage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenMessageStream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BlackboardHandlerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetMessages': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMessages,
                    request_deserializer=types__pb2.GetMessagesRequest.FromString,
                    response_serializer=types__pb2.MessageStack.SerializeToString,
            ),
            'SendMessage': grpc.unary_unary_rpc_method_handler(
                    servicer.SendMessage,
                    request_deserializer=types__pb2.Message.FromString,
                    response_serializer=types__pb2.Ack.SerializeToString,
            ),
            'OpenMessageStream': grpc.stream_stream_rpc_method_handler(
                    servicer.OpenMessageStream,
                    request_deserializer=types__pb2.Message.FromString,
                    response_serializer=types__pb2.Message.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'BlackboardHandler', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class BlackboardHandler(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetMessages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/BlackboardHandler/GetMessages',
            types__pb2.GetMessagesRequest.SerializeToString,
            types__pb2.MessageStack.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendMessage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/BlackboardHandler/SendMessage',
            types__pb2.Message.SerializeToString,
            types__pb2.Ack.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OpenMessageStream(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/BlackboardHandler/OpenMessageStream',
            types__pb2.Message.SerializeToString,
            types__pb2.Message.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# These are just aliases to make an agent implementation a little
# easier.  Only acceptable because the we should only ever need
# to update if we remove or add states.  My HOPE is that it's not
# necessary beyond the occasional tweak.

from . import types_pb2 as pb2
from enum import IntEnum

class Log(IntEnum):
    DEBUG = pb2.Log.Severity.Debug
    INFO = pb2.Log.Severity.Info
    WARNING = pb2.Log.Severity.Warning
    ERROR = pb2.Log.Severity.Error
    CRITICAL = pb2.Log.Severity.Critical

class AgentState(IntEnum):
    UNKNOWN = pb2.StatusUpdate.AgentState.Unknown
    OK = pb2.StatusUpdate.AgentState.OK
    REQUESTED = pb2.StatusUpdate.AgentState.Requested
    AWAITING_PROMISE = pb2.StatusUpdate.AgentState.AwaitingPromise
    WORKING  = pb2.StatusUpdate.AgentState.Working
    MISSING = pb2.StatusUpdate.AgentState.Missing
    ERROR = pb2.StatusUpdate.AgentState.Error
    COMPLETED = pb2.StatusUpdate.AgentState.Completed

class ResultType(IntEnum):
    RESULT = pb2.Result.ResultType.Result
    PARTIAL_RESULT = pb2.Result.ResultType.PartialResult
    NO_RESULT = pb2.Result.ResultType.NoResult

class PromiseState(IntEnum):
    PENDING = pb2.Promise.PromiseState.Pending
    FULFILLED  = pb2.Promise.PromiseState.Fulfilled
    FINAL = pb2.Promise.PromiseState.Final
    EMPTY = pb2.Promise.PromiseState.Empty



import sys

import grpc
import types_pb2 as pb2
import types_pb2_grpc as pb2_grpc

class BlackboardGRPCClient():
    def __init__(self, addr):
        # instantiate a channel
        # bind the client and the server
        self.addr = addr
        self.channel = grpc.insecure_channel(self.addr)
        self.stub = pb2_grpc.BlackboardHandlerStub(self.channel)

    def _get_messages(self, starting_at, filter_pings = True):
        msg_stack = self.stub.GetMessages(pb2.GetMessagesRequest(startingAt=starting_at, filterPings=filter_pings))
        return msg_stack.messages # iterable list of messages

    def _send_message(self, msg):
        self.stub.SendMessage(msg)

if __name__=="__main__":

    addr = sys.argv[1]
    
    b = BlackboardGRPCClient(addr)

    b.get_messages(0)


import argparse
from . import types_pb2 as pb2
from google.protobuf import json_format

# ----------------------------------------
# default_args provides a minimal argument parser to start an agen 
# We need to provide invocation bindings that automatically
# account for dispatches from controller.  All of the currently
# configured runners that can call python will pass the following
# to the CLI:
#
#     - Agent spec (in JSON) --spec={...}
#     - Blackboard addr      --bb="localhost:8080"
#
# If you're modifying this code, make sure the runner that will
# dispatch your job will pass these arguments (or the arguments that
# you specify).  But note that an AgentSpec and a Blackboard address
# are required to start any agent.
def default_args(agent_filename):
    parser = argparse.ArgumentParser(prog=agent_filename)
    parser.add_argument('--spec', help="full agent spec in JSON")
    parser.add_argument('--blackboard', help="hostname:port of the blackboard to connect to")
    args = parser.parse_args()
    
    agent_spec = json_format.Parse(args.spec, pb2.AgentSpec())
    return (agent_spec, args.blackboard)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: types.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0btypes.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x06\n\x04Ping\"\x07\n\x05Hello\"\t\n\x07Goodbye\"\xc3\x01\n\x0cStatusUpdate\x12\'\n\x05state\x18\x01 \x01(\x0e\x32\x18.StatusUpdate.AgentState\x12\x0f\n\x07message\x18\x02 \x01(\t\"y\n\nAgentState\x12\x0b\n\x07Unknown\x10\x00\x12\x06\n\x02OK\x10\x01\x12\r\n\tRequested\x10\x02\x12\x13\n\x0f\x41waitingPromise\x10\x03\x12\x0b\n\x07Working\x10\x04\x12\x0b\n\x07Missing\x10\x05\x12\t\n\x05\x45rror\x10\x06\x12\r\n\tCompleted\x10\x07\"\xab\x01\n\x07Promise\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0e\n\x06source\x18\x03 \x01(\t\x12$\n\x05state\x18\x04 \x01(\x0e\x32\x15.Promise.PromiseState\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\"@\n\x0cPromiseState\x12\x0b\n\x07Pending\x10\x00\x12\r\n\tFulfilled\x10\x01\x12\t\n\x05\x46inal\x10\x02\x12\t\n\x05\x45mpty\x10\x03\"\x80\x01\n\x06Result\x12 \n\x04type\x18\x01 \x01(\x0e\x32\x12.Result.ResultType\x12\x19\n\x07promise\x18\x02 \x01(\x0b\x32\x08.Promise\"9\n\nResultType\x12\n\n\x06Result\x10\x00\x12\x11\n\rPartialResult\x10\x01\x12\x0c\n\x08NoResult\x10\x02\"~\n\x03Log\x12\x1f\n\x08severity\x18\x01 \x01(\x0e\x32\r.Log.Severity\x12\x0f\n\x07message\x18\x02 \x01(\t\"E\n\x08Severity\x12\t\n\x05\x44\x65\x62ug\x10\x00\x12\x08\n\x04Info\x10\x01\x12\x0b\n\x07Warning\x10\x02\x12\t\n\x05\x45rror\x10\x03\x12\x0c\n\x08\x43ritical\x10\x04\"\x06\n\x04Halt\"\x12\n\x10HaltAndCatchFire\"\xbb\x03\n\x07Message\x12\n\n\x02id\x18\x01 \x01(\x05\x12\r\n\x05index\x18\x02 \x01(\x05\x12\x0e\n\x06source\x18\x03 \x01(\t\x12\x10\n\x08priority\x18\x04 \x01(\x05\x12,\n\x08timeSent\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08timeRecv\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\x04ping\x18\x07 \x01(\x0b\x32\x05.PingH\x00\x12\x17\n\x05hello\x18\x08 \x01(\x0b\x32\x06.HelloH\x00\x12\x1b\n\x07goodbye\x18\t \x01(\x0b\x32\x08.GoodbyeH\x00\x12%\n\x0cstatusUpdate\x18\n \x01(\x0b\x32\r.StatusUpdateH\x00\x12\x19\n\x06result\x18\x0b \x01(\x0b\x32\x07.ResultH\x00\x12\x13\n\x03log\x18\x0c \x01(\x0b\x32\x04.LogH\x00\x12\x15\n\x04halt\x18\r \x01(\x0b\x32\x05.HaltH\x00\x12-\n\x10haltAndCatchFire\x18\x0e \x01(\x0b\x32\x11.HaltAndCatchFireH\x00\x12!\n\x0b\x63reateAgent\x18\x0f \x01(\x0b\x32\n.AgentSpecH\x00\x42\n\n\x08\x63ontents\"\xc0\x02\n\tAgentSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06runner\x18\x02 \x01(\t\x12\x1f\n\nrunnerType\x18\x03 \x01(\x0e\x32\x0b.RunnerType\x12\x17\n\x0fpollInterval_ms\x18\x04 \x01(\x05\x12\x17\n\x0fpingInterval_ms\x18\x05 \x01(\x05\x12\x1a\n\x12\x61llowedPollRetries\x18\x06 \x01(\x05\x12\x1a\n\x12\x61llowedPingRetries\x18\x07 \x01(\x05\x12\x12\n\nentrypoint\x18\x08 \x01(\t\x12\x13\n\x0b\x62lackboards\x18\t \x03(\t\x12.\n\nattributes\x18\n \x03(\x0b\x32\x1a.AgentSpec.AttributesEntry\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\",\n\x0b\x43reateAgent\x12\x1d\n\tagentSpec\x18\x01 \x01(\x0b\x32\n.AgentSpec\"=\n\x12GetMessagesRequest\x12\x12\n\nstartingAt\x18\x01 \x01(\x05\x12\x13\n\x0b\x66ilterPings\x18\x02 \x01(\x08\"*\n\x0cMessageStack\x12\x1a\n\x08messages\x18\x01 \x03(\x0b\x32\x08.Message\"E\n\x03\x41\x63k\x12\x1b\n\x06status\x18\x01 \x01(\x0e\x32\x0b.Ack.Status\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x10\n\x06Status\x12\x06\n\x02OK\x10\x00*;\n\nRunnerType\x12\x0e\n\nRunnerBase\x10\x00\x12\x0c\n\x08RunnerGo\x10\x01\x12\x0f\n\x0bRunnerLocal\x10\x02\x32\x98\x01\n\x11\x42lackboardHandler\x12\x33\n\x0bGetMessages\x12\x13.GetMessagesRequest\x1a\r.MessageStack\"\x00\x12\x1f\n\x0bSendMessage\x12\x08.Message\x1a\x04.Ack\"\x00\x12-\n\x11OpenMessageStream\x12\x08.Message\x1a\x08.Message\"\x00(\x01\x30\x01\x42+Z)gitlab.com/sm-ai-team/sm-core-go/pkg/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'types_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z)gitlab.com/sm-ai-team/sm-core-go/pkg/core'
  _AGENTSPEC_ATTRIBUTESENTRY._options = None
  _AGENTSPEC_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _RUNNERTYPE._serialized_start=1728
  _RUNNERTYPE._serialized_end=1787
  _PING._serialized_start=48
  _PING._serialized_end=54
  _HELLO._serialized_start=56
  _HELLO._serialized_end=63
  _GOODBYE._serialized_start=65
  _GOODBYE._serialized_end=74
  _STATUSUPDATE._serialized_start=77
  _STATUSUPDATE._serialized_end=272
  _STATUSUPDATE_AGENTSTATE._serialized_start=151
  _STATUSUPDATE_AGENTSTATE._serialized_end=272
  _PROMISE._serialized_start=275
  _PROMISE._serialized_end=446
  _PROMISE_PROMISESTATE._serialized_start=382
  _PROMISE_PROMISESTATE._serialized_end=446
  _RESULT._serialized_start=449
  _RESULT._serialized_end=577
  _RESULT_RESULTTYPE._serialized_start=520
  _RESULT_RESULTTYPE._serialized_end=577
  _LOG._serialized_start=579
  _LOG._serialized_end=705
  _LOG_SEVERITY._serialized_start=636
  _LOG_SEVERITY._serialized_end=705
  _HALT._serialized_start=707
  _HALT._serialized_end=713
  _HALTANDCATCHFIRE._serialized_start=715
  _HALTANDCATCHFIRE._serialized_end=733
  _MESSAGE._serialized_start=736
  _MESSAGE._serialized_end=1179
  _AGENTSPEC._serialized_start=1182
  _AGENTSPEC._serialized_end=1502
  _AGENTSPEC_ATTRIBUTESENTRY._serialized_start=1453
  _AGENTSPEC_ATTRIBUTESENTRY._serialized_end=1502
  _CREATEAGENT._serialized_start=1504
  _CREATEAGENT._serialized_end=1548
  _GETMESSAGESREQUEST._serialized_start=1550
  _GETMESSAGESREQUEST._serialized_end=1611
  _MESSAGESTACK._serialized_start=1613
  _MESSAGESTACK._serialized_end=1655
  _ACK._serialized_start=1657
  _ACK._serialized_end=1726
  _ACK_STATUS._serialized_start=1710
  _ACK_STATUS._serialized_end=1726
  _BLACKBOARDHANDLER._serialized_start=1790
  _BLACKBOARDHANDLER._serialized_end=1942
# @@protoc_insertion_point(module_scope)
